import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;

import lejos.pc.comm.NXTComm;
import lejos.pc.comm.NXTCommException;
import lejos.pc.comm.NXTCommFactory;
import lejos.pc.comm.NXTInfo;

public class Map extends JPanel implements KeyListener, WindowListener {

	private ImageMap map;
	
	private DataOutputStream output;
	private DataInputStream input;
	private Send sendthread;
	private Receiver receivethread;

	public static final byte FORWARD = 0;
	public static final byte STOP = 1;
	public static final byte EXIT = 2;
	public static final byte LEFT = 3;
	public static final byte RIGHT = 4;
	public static final byte BACKWARD = 5;
	
	private class Receiver extends Thread {
		public boolean run = true;
		@Override
		public void run() {
			int bytes_valiable = -1;
			
			while(run) {
				try {
					bytes_valiable = input.available();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				if (bytes_valiable >= 0) {
					try {
						if (input.readByte() != '@') continue;
						int angle = input.readByte();
						float alpha = input.readFloat();
						float x = input.readFloat();
						int distance = input.readByte();
						float y = input.readFloat();
						
						// posicao do robo
						map.addPoint(x, y, Math.toRadians(alpha+90));
						
						// ponto do ultrasonico
						double sensor_ang = Math.toRadians(alpha+angle);
						double dx = Math.cos(sensor_ang)*distance;
						double dy = Math.sin(sensor_ang)*distance;
						map.addRead(x+dx, y+dy);
						
					} catch (IOException e1) {
						continue;
					}
				}
			}
		}
	}

	private class Send extends Thread {
		private int before;
		private int command;
		private boolean run = true;
		
		@Override
		public void run() {
			before = -1;
			command = STOP;
			
			while(run) {
				if (before != command) {
					try {
						System.out.println("Send cmd");
						output.write(command);
						output.flush();
						if (command == EXIT) run = false;
					} catch (IOException e1) {
						System.out.println("Erro send cmd");
						before = -1;
						send(command);
					}
					before = command;
				}
				
				try {
					Thread.sleep(50);
				} catch (InterruptedException e) {

				}
			}
		}
		public void send(int cmd) {
			command = cmd;
		}
	}
	
	public Map (DataInputStream in, DataOutputStream out) {
		JFrame frame = new JFrame("Mapa MAC0318");
		receivethread= new Receiver();
		sendthread = new Send();
		
		output = out;
		input = in;

		receivethread.start();
		sendthread.start();
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setLayout(new BorderLayout());
        map = new ImageMap();
		frame.add(this.map);
		frame.setSize(800, 800);
		frame.setVisible(true);
		
		frame.setFocusable(true);
		frame.requestFocusInWindow();
		frame.addKeyListener(this);
		frame.addWindowListener(this);
		
		
		String text = "1,2,3 - Change view mode.\n";
		text += "s - Save image.\n";
		text += "l - Show trace.\n";
		text += "c - Clean map.\n";
		JOptionPane.showMessageDialog(null, text);
	}
	
    public void addPoint(Point3D p) {
    		map.addPoint(p);
	}

	@Override
	public void keyPressed(KeyEvent e) {
		char input = e.getKeyChar();
		if (input == '1') map.setVisual(0);
		else if (input == '2') map.setVisual(1);
		else if (input == '3') map.setVisual(2);
		else if (input == 'l') map.showLine();
		else if (input == 's') map.save();
		else if (input == 'c') map.clean();
		
		 switch(e.getKeyCode()) { 
	        case KeyEvent.VK_UP:
	        		sendthread.send(FORWARD);
	            break;
	        case KeyEvent.VK_DOWN:
	        		sendthread.send(BACKWARD);
	            break;
	        case KeyEvent.VK_LEFT:
	        		sendthread.send(LEFT);
	            break;
	        case KeyEvent.VK_RIGHT :
	        		sendthread.send(RIGHT);
	            break;
	    }			 
	}

	@Override
	public void keyReleased(KeyEvent arg0) {
		sendthread.send(STOP);
	}

	@Override
	public void keyTyped(KeyEvent arg0) {		
	}

	public static void main(String[] args) throws NXTCommException, IOException {
		NXTComm nxtComm = NXTCommFactory.createNXTComm(NXTCommFactory.BLUETOOTH); 
		NXTInfo[] nxtInfo = nxtComm.search("NXT8"); //find brick with NXT_ID by doing a Bluetooth inquiry
		if (nxtInfo.length == 0) { // failed to find a brick with the ID
			System.err.println("NO NXT found");
			System.exit(1);
		}
		if (!nxtComm.open(nxtInfo[0])) { // the brick was found but a connection could not be establish
			System.err.println("Failed to open NXT");
			System.exit(1);
		}
		
		final DataInputStream input = new DataInputStream(nxtComm.getInputStream()); // open data input stream 
		final DataOutputStream output = new DataOutputStream(nxtComm.getOutputStream()); // open data output stream
		
		SwingUtilities.invokeLater(new Runnable() {
            public void run() {
            	new Map(input, output);
            }
        });
		
	}

	@Override
	public void windowOpened(WindowEvent e) {
		
	}

	@Override
	public void windowClosing(WindowEvent e) {
		System.err.println("Fechando...");
		receivethread.run = false;
		sendthread.send(EXIT);
		try {
			receivethread.join();
			sendthread.join();
		} catch (InterruptedException e1) {
			System.out.println("Nao foi possivel finalizar as threads...");
		}
		
	}

	@Override
	public void windowClosed(WindowEvent e) {
	}

	@Override
	public void windowIconified(WindowEvent e) {
		
	}

	@Override
	public void windowDeiconified(WindowEvent e) {
		
	}

	@Override
	public void windowActivated(WindowEvent e) {
		
	}

	@Override
	public void windowDeactivated(WindowEvent e) {		
	}
}
